// waveshare-shield-adda-11010.js
//
// Copyright (c) 2020-2022 Spring City Solutions LLC and other contributors
// Licensed under the Apache License, Version 2.0
// WAVESHARE (tm) is a trademark of Shenzhen Weixue Electronic Co., Ltd.
//
// Hardware Links:
//
// Board Manufacturer Store:
// https://www.waveshare.com/High-Precision-AD-DA-Board.htm
//
// Board Manufacturer Wiki:
// https://www.waveshare.com/wiki/High-Precision_AD/DA_Board
//
// Links for ADS1256 chip:
// https://www.ti.com/product/ADS1256
// https://www.ti.com/lit/gpn/ads1256
// Book "Fundamentals of Precision ADC Noise Analysis" https://www.ti.com/lit/pdf/slyy192
// Journal Series "How delta-sigma ADCs work" https://www.ti.com/lit/pdf/slyt423 https://www.ti.com/lit/pdf/slyt438
// App Note "Digital Filter Types in Delta-Sigma ADCs" https://www.ti.com/lit/pdf/sbaa230
//
// Link for LM285 chip, connected to the ADS1256:
// https://www.ti.com/product/LM285-2.5-N
// https://www.ti.com/lit/gpn/lm285-2.5-n
//
// Link for DAC8532 chip:
// https://www.ti.com/product/DAC8532
// https://www.ti.com/lit/gpn/dac8532
//
// Spring City Solutions Links:
//
// Spring City Solutions Node-RED project page:
// https://www.springcitysolutions.com/nodered
//
// Spring City Solutions page for this node:
// https://www.springcitysolutions.com/nodered-waveshare-adda-shield
//
// Spring City Solutions Node-RED project email:
// nodered@springcitysolutions.com
//
// Spring City Solutions Gitlab for this node:
// https://gitlab.com/SpringCitySolutionsLLC/waveshare-shield-adda-11010
//
// Patreon Page:
// https://www.patreon.com/springcitysolutions_nodered
//
// Software Links:
//
// Spring City Solutions Gitlab for this node:
// https://gitlab.com/SpringCitySolutionsLLC/waveshare-shield-adda-11010
//
// Doxygen docs for this node autogenerated by Gitlab CI/CD:
// https://springcitysolutionsllc.gitlab.io/waveshare-shield-adda-11010/index.html
//
// npmjs for this node:
// https://npmjs.org/package/node-red-contrib-waveshare-shield-adda-11010
//
// Node-RED flows for this node:
// https://flows.nodered.org/node/node-red-contrib-waveshare-shield-adda-11010
//
// Documentation Links:
//
// Gitlab wiki for this node (the master copy of list of links is here):
// https://gitlab.com/SpringCitySolutionsLLC/waveshare-shield-adda-11010/-/wikis/home
//
// Waveshare's libraries for this hardware:
// https://github.com/waveshare/High-Precision-AD-DA-Board
//
// Youtube video "How to set up":
// TODO
//
// Youtube video "How to use":
// TODO
//
// Youtube video "Testing Results":
// TODO
//
module.exports = function (RED) {
  'use strict'

  const path = require('path')

  // Static security scans trigger on child_process.
  // I am careful to sanitize user input so we are OK.
  const execSync = require('child_process').execSync

  const testCommand = path.join(__dirname, 'library-test.py')
  const adcCommand = path.join(__dirname, 'ads1256.py')
  const dacCommand = path.join(__dirname, 'dac8532.py')
  const gpioCommand = path.join(__dirname, 'gpio.py')

  let allOK = true

  try {
    execSync(testCommand)
  } catch (err) {
    allOK = false
    RED.log.warn('waveshare-shield-adda-11010 : ' + RED._('waveshare-shield-adda-11010.errors.libnotfound'))
  }

  // the magic to make python print stuff immediately
  process.env.PYTHONUNBUFFERED = 1

  function analoginput (n) {
    RED.nodes.createNode(this, n)
    this.out = n.out || 'out'
    const node = this
    let out = ''
    this.scale = n.scale
    this.gain = n.gain
    this.port = n.port
    this.rate = n.rate
    this.buffer = n.buffer
    this.sdcs = n.sdcs

    function inputlistener (msg, send, done) {
      try {
        switch (node.port) {
          case 'fsc':
            out = execSync(adcCommand + ' --mode calibrationFSC').toString()
            break
          case 'ofc':
            out = execSync(adcCommand + ' --mode calibrationOFC').toString()
            break
          default:
            out = execSync(adcCommand + ' --mode analog --port ' + node.port + ' --gain ' + node.gain + ' --rate ' + node.rate + ' --buffer ' + node.buffer + ' --sdcs ' + node.sdcs).toString()
        }
      } catch (err) {
        allOK = false
        node.status({ fill: 'red', shape: 'ring', text: 'waveshare-shield-adda-11010.status.not-running' })
        RED.log.warn('waveshare-shield-adda-11010 : ' + RED._('waveshare-shield-adda-11010.errors.ads1256failure'))
      }
      const scaledOut = Number(out) / 1677721.5
      if (scaledOut < 0) {
        allOK = false
        node.status({ fill: 'red', shape: 'ring', text: 'waveshare-shield-adda-11010.status.not-running' })
        RED.log.warn('waveshare-shield-adda-11010 : ' + RED._('waveshare-shield-adda-11010.errors.ads1256failure'))
      } else {
        if (node.scale === 'voltage') {
          msg.payload = Number(out) / 1677721.5 / node.gain
        }
        if (node.scale === 'millivolt') {
          msg.payload = scaledOut * 1000.0 / node.gain
        }
        if (node.scale === 'fraction') {
          msg.payload = Number(out) / 8388607.0
        }
        if (node.scale === 'percentage') {
          msg.payload = Number(out) / 83886.07
        }
        if (node.scale === 'counts') {
          msg.payload = Number(out)
        }
        if (node.port === 'fsc') {
          msg.payload = Number(out)
        }
        if (node.port === 'ofc') {
          msg.payload = Number(out)
        }
        node.send(msg)
        if (RED.settings.verbose) { node.log('analog out: ' + msg.payload) }
        node.status({ fill: 'green', shape: 'dot', text: msg.payload.toString() })
      }
      if (done) { done() }
    }
    if (allOK === true) {
      node.running = true
      node.on('input', inputlistener)
    } else {
      node.status({ fill: 'grey', shape: 'dot', text: 'waveshare-shield-adda-11010.status.not-available' })
      node.on('input', function (msg) {
        node.status({ fill: 'grey', shape: 'dot', text: RED._('waveshare-shield-adda-11010.status.na', { value: msg.payload.toString() }) })
      })
    }

    node.on('close', function (done) {
      node.status({ fill: 'grey', shape: 'ring', text: 'waveshare-shield-adda-11010.status.closed' })
      done()
    })
  }
  RED.nodes.registerType('analoginput', analoginput)

  function analogoutput (n) {
    RED.nodes.createNode(this, n)
    this.scale = n.scale
    this.port = n.port
    this.vref = n.vref
    this.out = n.out || 'out'
    const node = this

    function inputlistener (msg, send, done) {
      let out = Number(msg.payload)
      if (node.scale === 'voltage') {
        if (node.vref === 'vref-33') { out = 19859.4 * out };
        if (node.vref === 'vref-50') { out = 13107.2 * out };
      }
      if (node.scale === 'millivolt') {
        if (node.vref === 'vref-33') { out = 19859400 * out };
        if (node.vref === 'vref-50') { out = 13107200 * out };
      }
      if (node.scale === 'fraction') {
        out = out * 65535
      }
      if (node.scale === 'percentage') {
        out = out * 655.35
      }
      if (out < 0) { out = 0 };
      if (out > 65535) { out = 65535 };
      out = parseInt(out).toString()
      if (RED.settings.verbose) { node.log('analogoutput out: ' + out) }
      node.status({ fill: 'green', shape: 'dot', text: msg.payload.toString() })
      try {
        execSync(dacCommand + ' --port ' + node.port + ' --output ' + out)
      } catch (err) {
        allOK = false
        node.status({ fill: 'red', shape: 'ring', text: 'waveshare-shield-adda-11010.status.not-running' })
        RED.log.warn('waveshare-shield-adda-11010 : ' + RED._('waveshare-shield-adda-11010.errors.dac8532failure'))
      }
      if (done) { done() }
    }

    if (allOK === true) {
      node.running = true
      node.on('input', inputlistener)
    } else {
      node.status({ fill: 'grey', shape: 'dot', text: 'waveshare-shield-adda-11010.status.not-available' })
      node.on('input', function (msg) {
        node.status({ fill: 'grey', shape: 'dot', text: RED._('waveshare-shield-adda-11010.status.na', { value: msg.payload.toString() }) })
      })
    }

    node.on('close', function (done) {
      node.status({ fill: 'grey', shape: 'ring', text: 'waveshare-shield-adda-11010.status.closed' })
      execSync(dacCommand + ' --port A' + ' --output 0')
      execSync(dacCommand + ' --port B' + ' --output 0')
      done()
    })
  }

  RED.nodes.registerType('analogoutput', analogoutput)

  function gpioinput (n) {
    RED.nodes.createNode(this, n)
    this.out = n.out || 'out'
    const node = this
    let out = ''
    this.port = n.port

    function inputlistener (msg, send, done) {
      try {
        switch (node.port) {
          case 'D0':
          case 'D1':
          case 'D2':
          case 'D3':
            out = execSync(adcCommand + ' --mode digitalIn --pin ' + node.port).toString()
            break
          case 'P22':
          case 'P23':
          case 'P24':
          case 'P25':
            out = execSync(gpioCommand + ' --pin ' + node.port).toString()
            break
        }
      } catch (err) {
        allOK = false
        node.status({ fill: 'red', shape: 'ring', text: 'waveshare-shield-adda-11010.status.not-running' })
        RED.log.warn('waveshare-shield-adda-11010 : ' + RED._('waveshare-shield-adda-11010.errors.gpiofailure'))
      }
      msg.payload = Number(out)
      node.send(msg)
      if (RED.settings.verbose) { node.log('gpio input: ' + msg.payload) }
      node.status({ fill: 'green', shape: 'dot', text: msg.payload.toString() })
      if (done) { done() }
    }
    if (allOK === true) {
      node.running = true
      node.on('input', inputlistener)
    } else {
      node.status({ fill: 'grey', shape: 'dot', text: 'waveshare-shield-adda-11010.status.not-available' })
      node.on('input', function (msg) {
        node.status({ fill: 'grey', shape: 'dot', text: RED._('waveshare-shield-adda-11010.status.na', { value: msg.payload.toString() }) })
      })
    }
    node.on('close', function (done) {
      node.status({ fill: 'grey', shape: 'ring', text: 'waveshare-shield-adda-11010.status.closed' })
      done()
    })
  }
  RED.nodes.registerType('gpioinput', gpioinput)

  function gpiooutput (n) {
    RED.nodes.createNode(this, n)
    this.scale = n.scale
    this.out = n.out || 'out'
    const node = this
    this.port = n.port

    function inputlistener (msg, send, done) {
      let out = Number(msg.payload)
      if (out < 0) { out = 0 };
      if (out > 1) { out = 1 };
      out = parseInt(out).toString()
      if (RED.settings.verbose) { node.log('gpio output: ' + out) }
      node.status({ fill: 'green', shape: 'dot', text: msg.payload.toString() })
      try {
        switch (node.port) {
          case 'D0':
          case 'D1':
          case 'D2':
          case 'D3':
            execSync(adcCommand + ' --mode digitalOut --pin ' + node.port + ' --output ' + out)
            break
          case 'P22':
          case 'P23':
          case 'P24':
          case 'P25':
            execSync(gpioCommand + ' --pin ' + node.port + ' --output ' + out)
            break
        }
      } catch (err) {
        allOK = false
        node.status({ fill: 'red', shape: 'ring', text: 'waveshare-shield-adda-11010.status.not-running' })
        RED.log.warn('waveshare-shield-adda-11010 : ' + RED._('waveshare-shield-adda-11010.errors.gpiofailure'))
      }
      if (done) { done() }
    }

    if (allOK === true) {
      node.running = true
      node.on('input', inputlistener)
    } else {
      node.status({ fill: 'grey', shape: 'dot', text: 'waveshare-shield-adda-11010.status.not-available' })
      node.on('input', function (msg) {
        node.status({ fill: 'grey', shape: 'dot', text: RED._('waveshare-shield-adda-11010.status.na', { value: msg.payload.toString() }) })
      })
    }

    node.on('close', function (done) {
      node.status({ fill: 'grey', shape: 'ring', text: 'waveshare-shield-adda-11010.status.closed' })
      done()
    })
  }
  RED.nodes.registerType('gpiooutput', gpiooutput)
}
